#include "qf.h"

template <class T>
T QF<T>::Diff(T x,T dx,int n,RefFunc f)
{
	if (n==0) return f(x);
	T fxpdx=Diff(x+0.5*dx,dx,n-1,f);
	T fxmdx=Diff(x-0.5*dx,dx,n-1,f);
	return ((fxpdx-fxmdx)/dx);
}

template <class T>
T QF<T>::DiffMax(T a,T b,T dx,int n,RefFunc f)
{
	double x=a;
	double max=0;
	while (x<=b)
	{
		T d=Diff(x,dx,n,f);
		if (d<0) d=-d;
		if (d>max) max=d;
		x+=dx;
	}
	return max;
}

template <class T>
T QF<T>::Rect(T a,T b,int n,RefFunc f)
{
	T h=(b-a)/(T)n;
	T s=0;
	T a0=a+h/2;

	for (int i=0; i<n; i++)
	{
		s+=f(a0+(T)i*h);
	}
	return s*h;
}

template <class T>
T QF<T>::Trapez(T a,T b,int n,RefFunc f)
{
	T h=(b-a)/(T)n;
	T s=(f(a)+f(b))/2;

	for (int i=1; i<n; i++)
	{
		s+=f(a+(T)i*h);
	}
	return s*h;
}

template <class T>
T QF<T>::Simpson(T a,T b,int n,RefFunc f)
{
	T h=(b-a)/(T)n;
	T s=(f(a)+f(b))/2;

	for (int i=1; i<n; i++)
	{
		s+=f(a+(T)i*h)+2*f(a+(T)i*h-h/2);
	}
	s+=2*f(a+(T)n*h-h/2);
	return s*h/3;
}

template <class T>
T QF<T>::Newton(T a,T b,int n,RefFunc f)
{
	T h=(b-a)/(T)n;
	T s=f(a)+f(b)+3*(f(a+h/3)+f(a+2*h/3));

	for (int i=1; i<n; i++)
	{
		s+=2*f(a+(T)i*h)+3*(f(a+(T)i*h+h/3)+f(a+(T)i*h+2*h/3));
	}
	return s*h/8;
}

template <class T>
T QF<T>::Rect(T a,T b,T eps,RefFunc f)
{		
	T M2=DiffMax(a,b,eps,2,f);
	nelem=(int)sqrt( (b-a)*(b-a)*(b-a)*M2/24/eps)+1;
	return Rect(a,b,nelem,f);
}

template <class T>
T QF<T>::Trapez(T a,T b,T eps,RefFunc f)
{		
	T M2=DiffMax(a,b,eps,2,f);
	nelem=(int)sqrt( (b-a)*(b-a)*(b-a)*M2/12/eps)+1;
	return Trapez(a,b,nelem,f);
}

template <class T>
T QF<T>::Simpson(T a,T b,T eps,RefFunc f)
{		
	T M4=DiffMax(a,b,eps,4,f);
	nelem=(int)sqrt(sqrt( (b-a)*(b-a)*(b-a)*(b-a)*(b-a)*M4/2880/eps))+1;
	return Simpson(a,b,nelem,f);
}

template <class T>
T QF<T>::Newton(T a,T b,T eps,RefFunc f)
{		
	T M4=DiffMax(a,b,eps,4,f);
	nelem=(int)sqrt(sqrt( (b-a)*(b-a)*(b-a)*(b-a)*(b-a)*M4/6480/eps))+1;
	return Newton(a,b,nelem,f);
}